import { TextDocument, Position, CompletionList, Hover, Range, SymbolInformation, Diagnostic, Location, DocumentHighlight, CodeActionContext, Command, WorkspaceEdit } from 'vscode-languageserver-types';
export declare type Stylesheet = {};
export interface Color {
    red: number;
    blue: number;
    green: number;
    alpha: number;
}
export interface ColorInformation {
    range: Range;
    color: Color;
}
export interface LanguageService {
    configure(raw: LanguageSettings): void;
    doValidation(document: TextDocument, stylesheet: Stylesheet, documentSettings?: LanguageSettings): Diagnostic[];
    parseStylesheet(document: TextDocument): Stylesheet;
    doComplete(document: TextDocument, position: Position, stylesheet: Stylesheet): CompletionList;
    doHover(document: TextDocument, position: Position, stylesheet: Stylesheet): Hover;
    findDefinition(document: TextDocument, position: Position, stylesheet: Stylesheet): Location;
    findReferences(document: TextDocument, position: Position, stylesheet: Stylesheet): Location[];
    findDocumentHighlights(document: TextDocument, position: Position, stylesheet: Stylesheet): DocumentHighlight[];
    findDocumentSymbols(document: TextDocument, stylesheet: Stylesheet): SymbolInformation[];
    doCodeActions(document: TextDocument, range: Range, context: CodeActionContext, stylesheet: Stylesheet): Command[];
    /** deprecated, use findDocumentColors instead */
    findColorSymbols(document: TextDocument, stylesheet: Stylesheet): Range[];
    findDocumentColors(document: TextDocument, stylesheet: Stylesheet): ColorInformation[];
    doRename(document: TextDocument, position: Position, newName: string, stylesheet: Stylesheet): WorkspaceEdit;
}
export declare type LintSettings = {
    [key: string]: string;
};
export interface LanguageSettings {
    validate?: boolean;
    lint?: LintSettings;
}
export declare function getCSSLanguageService(): LanguageService;
export declare function getSCSSLanguageService(): LanguageService;
export declare function getLESSLanguageService(): LanguageService;
